@@ -1,12 +1,18 @@
 from onegov.file.collection import FileCollection, FileSetCollection
 from onegov.file.integration import DepotApp
-from onegov.file.models import File, FileSet, AssociatedFiles
+from onegov.file.models import (
+    File,
+    FileSet,
+    AssociatedFiles,
+    SearchableFile
+)
 
-__all__ = [
+__all__ = (
     'AssociatedFiles',
     'DepotApp',
     'File',
     'FileCollection',
     'FileSet',
-    'FileSetCollection'
-]
+    'FileSetCollection',
+    'SearchableFile',
+)
