@@ -1,19 +1,23 @@
-import hashlib
+import pdftotext
 
 from depot.fields.upload import UploadedFile
 from depot.io import utils
 from depot.io.interfaces import FileStorage
 from depot.io.utils import INMEMORY_FILESIZE
+from io import BytesIO
 from onegov.core.html import sanitize_svg
-from onegov.file.utils import IMAGE_MIME_TYPES, get_svg_size, get_image_size
+from onegov.file.utils import digest
+from onegov.file.utils import get_image_size
+from onegov.file.utils import get_svg_size
+from onegov.file.utils import IMAGE_MIME_TYPES
+from onegov.file.utils import word_count
 from PIL import Image
 from tempfile import SpooledTemporaryFile
-from io import BytesIO
 
 
-IMAGE_MAX_SIZE = 1024
-IMAGE_QUALITY = 90
-CHECKSUM_FUNCTION = hashlib.md5
+IMAGE_MAX_SIZE = 2048
+IMAGE_QUALITY = 85
+CHECKSUM_FUNCTION = 'md5'
 
 
 def get_svg_size_or_default(content):
@@ -47,12 +51,7 @@ def limit_and_store_image_size(file, content, content_type):
 
 
 def calculate_checksum(content):
-    checksum = CHECKSUM_FUNCTION()
-
-    for chunk in iter(lambda: content.read(4096), b""):
-        checksum.update(chunk)
-
-    return checksum.hexdigest()
+    return digest(content, type=CHECKSUM_FUNCTION)
 
 
 def store_checksum(file, content, content_type):
@@ -67,12 +66,28 @@ def sanitize_svg_images(file, content, content_type):
     return content
 
 
+def extract_pdf_info(content):
+    pages = pdftotext.PDF(content)
+    return len(pages), '\n'.join(pages).strip(' \t\r\n').replace('\0', '')
+
+
+def store_extract_and_pages(file, content, content_type):
+    if content_type == 'application/pdf':
+        pages, file.extract = extract_pdf_info(content)
+
+        file.stats = {
+            'pages': pages,
+            'words': word_count(file.extract)
+        }
+
+
 class ProcessedUploadedFile(UploadedFile):
 
     processors = (
         store_checksum,
         limit_and_store_image_size,
-        sanitize_svg_images
+        sanitize_svg_images,
+        store_extract_and_pages,
     )
 
     def process_content(self, content, filename=None, content_type=None):
