@@ -1,10 +1,9 @@
 from datetime import timedelta
 from depot.io.utils import file_from_content
-from onegov.file.attachments import calculate_checksum
 from onegov.file.models import File, FileSet
-from onegov.file.utils import as_fileintent
+from onegov.file.utils import as_fileintent, digest
 from sedate import utcnow
-from sqlalchemy import and_
+from sqlalchemy import and_, text, or_
 
 
 class FileCollection(object):
@@ -133,9 +132,92 @@ class FileCollection(object):
     def by_content(self, content):
         """ Returns a query that matches the given content (may be more than
         one record).
+
+        """
+        file = file_from_content(content)
+
+        # we need to look up two checksums, the one of the file stored and
+        # possibly the one it had before signing
+        #
+        # XXX maybe it makes sense to combine those into a data structure
+        # that holds all kinds of digests a file is known under
+
+        # checksum
+        md5 = digest(file, 'md5')
+
+        # old_digest of signature_metadata
+        sha = digest(file, 'sha256')
+
+        return self.query().filter(or_(
+            File.checksum == md5,
+            File.signature_metadata['old_digest'].astext == sha
+        ))
+
+    def by_content_type(self, content_type):
+        """ Returns a query that matches the given MIME content type (may be
+        more than one record).
+
         """
 
-        return self.by_checksum(calculate_checksum(file_from_content(content)))
+        return self.query().filter(
+            text("reference->>'content_type' = :content_type").bindparams(
+                content_type=content_type
+            )
+        )
+
+    def by_signature_digest(self, digest):
+        """ Returns a query that matches the given digest in the signature
+        metadata. In other words, given a digest this function will find
+        signed files that match the digest - either before or after signing.
+
+        Unsigned files are ignored.
+
+        The digest is expected to be a SHA256 hex.
+
+        """
+
+        return self.query().filter_by(signed=True).filter(
+            or_(
+                text("signature_metadata->>'old_digest' = :digest").bindparams(
+                    digest=digest
+                ),
+                text("signature_metadata->>'new_digest' = :digest").bindparams(
+                    digest=digest
+                )
+            )
+        )
+
+    def locate_signature_metadata(self, digest):
+        """ Looks for the given digest in the files table - if that doesn't
+        work it will go through the audit trail (i.e. the chat messages) and
+        see if the digest can be found there.
+
+        If this database was ever used to sign a file with the given digest,
+        or if a file that was signed had the given digest, this function
+        will find it - barring manual database manipulation in the messages
+        log.
+
+        """
+
+        match = self.by_signature_digest(digest).with_entities(
+            File.signature_metadata).first()
+
+        if match:
+            return match.signature_metadata
+
+        from onegov.chat import Message  # circular import
+
+        match = self.session.query(Message).filter_by(type='file').filter(or_(
+            text(
+                "meta->'action_metadata'->>'old_digest' = :digest"
+            ).bindparams(digest=digest),
+            text(
+                "meta->'action_metadata'->>'new_digest' = :digest"
+            ).bindparams(digest=digest)
+        )).with_entities(Message.meta).first()
+
+        if match:
+            return match.meta['action_metadata']
 
 
 class FileSetCollection(object):
