@@ -1,4 +1,6 @@
 import json
+import sedate
+import isodate
 
 from depot.fields.sqlalchemy import UploadedFileField as UploadedFileFieldBase
 from onegov.core.crypto import random_token
@@ -10,9 +12,16 @@ from onegov.core.utils import normalize_for_url
 from onegov.file.attachments import ProcessedUploadedFile
 from onegov.file.filters import OnlyIfImage, WithThumbnailFilter
 from onegov.file.filters import OnlyIfPDF, WithPDFThumbnailFilter
+from onegov.file.utils import extension_for_content_type
+from onegov.search import ORMSearchable
 from pathlib import Path
 from sqlalchemy import Boolean, Column, Index, Text
+from sqlalchemy import case
 from sqlalchemy import event
+from sqlalchemy import text
+from sqlalchemy import type_coerce
+from sqlalchemy.ext.hybrid import hybrid_property
+from sqlalchemy.orm import deferred
 from sqlalchemy.orm import object_session, Session
 from sqlalchemy.orm.attributes import flag_modified
 from sqlalchemy_utils import observes
@@ -36,6 +45,32 @@ class UploadedFileField(UploadedFileFieldBase):
             return self._upload_type(value)
 
 
+class SearchableFile(ORMSearchable):
+    """ Files are not made available for elasticsearch by default. This is
+    for security reasons - files are public by default but one has to know
+    the url (a very long id).
+
+    Search might lead to a disclosure of all files, which is why files
+    can only be searched if they are of a different polymorphic subclass
+    and use this mixin.
+
+    """
+
+    es_properties = {
+        'name': {'type': 'text'},
+        'note': {'type': 'localized'},
+        'extract': {'type': 'localized'}
+    }
+
+    @property
+    def es_suggestion(self):
+        return self.name
+
+    @property
+    def es_public(self):
+        return self.published
+
+
 class File(Base, Associable, TimestampMixin):
     """ A general file (image, document, pdf, etc), referenced in the database.
 
@@ -78,6 +113,17 @@ class File(Base, Associable, TimestampMixin):
     #: something we ignore)
     signed = Column(Boolean, nullable=False, default=False)
 
+    #: the metadata of the signature - this should include the following
+    #: data::
+    #:
+    #:  - old_digest: The sha-256 hash before the file was signed
+    #:  - new_digest: The sha-256 hash after the file was signed
+    #:  - signee: The username of the user that signed the document
+    #:  - timestamp: The time the document was signed in UTC
+    #:  - request_id: A unique identifier by the signing service
+    #:
+    signature_metadata = deferred(Column(JSON, nullable=True))
+
     #: the type of the file, this can be used to create custom polymorphic
     #: subclasses. See `<http://docs.sqlalchemy.org/en/improve_toc/
     #: orm/extensions/declarative/inheritance.html>`_.
@@ -95,7 +141,7 @@ class File(Base, Associable, TimestampMixin):
                 # note, the thumbnail configuration should not be changed
                 # anywhere but here - for consistency.
                 WithThumbnailFilter(
-                    name='small', size=(256, 256), format='png'
+                    name='small', size=(512, 512), format='png'
                 )
             ),
             OnlyIfPDF(
@@ -115,6 +161,16 @@ class File(Base, Associable, TimestampMixin):
     #: strictly a check of file duplicates, not protection against tampering
     checksum = Column(Text, nullable=True, index=True)
 
+    #: the content of the given file as text, if it can be extracted
+    #: (it is important that this column be loaded deferred by default, lest
+    #: we load massive amounts of text on simple queries)
+    extract = deferred(Column(Text, nullable=True))
+
+    #: statistics around the extract (number of pages, words, etc.)
+    #: those are usually set during file upload (as some information is
+    #: lost afterwards)
+    stats = deferred(Column(JSON, nullable=True))
+
     __mapper_args__ = {
         'polymorphic_on': 'type'
     }
@@ -123,9 +179,41 @@ class File(Base, Associable, TimestampMixin):
         Index('files_by_type_and_order', 'type', 'order'),
     )
 
+    @hybrid_property
+    def signature_timestamp(self):
+        if self.signed:
+            return sedate.replace_timezone(
+                isodate.parse_datetime(self.signature_metadata['timestamp']),
+                'UTC'
+            )
+
+    @signature_timestamp.expression
+    def signature_timestamp(self):
+        return type_coerce(case(
+            [(
+                File.signed == True,
+                text("""
+                    (
+                        to_timestamp(
+                            signature_metadata->>'timestamp',
+                            'YYYY-MM-DD"T"HH24:MI:SS.US'
+                        )::timestamp without time zone
+                    )
+                """)
+            )],
+            else_=text('NULL')
+        ), UTCDateTime)
+
     @observes('reference')
     def reference_observer(self, reference):
-        self.checksum = self.reference.get('checksum')
+        if 'checksum' in self.reference:
+            self.checksum = self.reference.pop('checksum')
+
+        if 'extract' in self.reference:
+            self.extract = self.reference.pop('extract')
+
+        if 'stats' in self.reference:
+            self.stats = self.reference.pop('stats')
 
     @observes('name')
     def name_observer(self, name):
@@ -144,6 +232,21 @@ class File(Base, Associable, TimestampMixin):
 
         return self.virtual_file_id or self.reference.file_id
 
+    @property
+    def claimed_extension(self):
+        """ Returns the extension as defined by the file name or by the
+        content type (whatever is found first in this order).
+
+        Note that this extension could therefore not be correct. It is mainly
+        meant for display purposes.
+
+        If you need to know the type of a file you should use the
+        content type stored on the reference.
+
+        """
+        return extension_for_content_type(
+            self.reference['content_type'], self.name)
+
     def get_thumbnail_id(self, size):
         """ Returns the thumbnail id with the given size (e.g. 'small').
 
@@ -166,6 +269,12 @@ class File(Base, Associable, TimestampMixin):
         """
         assert set(options.keys()).issubset({'content_type', 'filename'})
 
+        if not hasattr(self.reference.file, '_metadata_path'):
+            raise NotImplementedError(
+                "The current depot storage backend does not support "
+                "in-place metadata updates"
+            )
+
         path = Path(self.reference.file._metadata_path)
 
         # store the pending metadata on the session to commit them later
@@ -200,6 +309,6 @@ def update_metadata_after_commit(session):
 
 
 @event.listens_for(Session, 'after_soft_rollback')
-def discard_metadata_on_rollback(session):
+def discard_metadata_on_rollback(session, previous_transaction):
     if 'pending_metadata_changes' in session.info:
         del session.info['pending_metadata_changes']
