@@ -1,7 +1,10 @@
+import hashlib
 import magic
+import os
 
-from contextlib import suppress
+from contextlib import contextmanager, suppress
 from depot.io.utils import FileIntent
+from mimetypes import guess_extension
 from io import IOBase, BytesIO, UnsupportedOperation
 from lxml import etree
 from PIL import Image
@@ -79,10 +82,73 @@ def get_svg_size(svg):
     return root.get('width'), root.get('height')
 
 
+def extension_for_content_type(content_type, filename=None):
+    """ Gets the extension for the given content type. Note that this is
+    *meant for display only*. A file claiming to be a PDF might not be one,
+    but this function would not let you know that.
+
+    """
+
+    if filename is not None:
+        ext = filename.split('.')[-1][:4].lower()
+    else:
+        ext = (guess_extension(content_type, strict=False) or '')
+
+    return ext.strip('. ')
+
+
 def get_image_size(image):
     return tuple('{}px'.format(d) for d in image.size)
 
 
+def digest(fileobj, type='sha256', chunksize=4096):
+    with suppress(UnsupportedOperation):
+        fileobj.seek(0)
+
+    digest = getattr(hashlib, type)()
+
+    for chunk in iter(lambda: fileobj.read(chunksize), b''):
+        digest.update(chunk)
+
+    return digest.hexdigest()
+
+
+def word_count(text):
+    """ The word-count of the given text. Goes through the string exactly
+    once and has constant memory usage. Not super sophisticated though.
+
+    """
+    if not text:
+        return 0
+
+    count = 0
+    inside_word = False
+
+    for char in text:
+        if char.isspace():
+            inside_word = False
+        elif not inside_word:
+            count += 1
+            inside_word = True
+
+    return count
+
+
+def name_without_extension(name):
+    if '.' in name.rstrip()[-5:]:
+        return name.rsplit('.', 1)[0]
+
+    return name
+
+
+@contextmanager
+def current_dir(dir):
+    previous = os.getcwd()
+    os.chdir(dir)
+    yield
+    os.chdir(previous)
+
+
 # we don't support *all* the image types PIL supports
 EXCLUDED_IMAGE_TYPES = {'application/pdf'}
 IMAGE_MIME_TYPES = get_supported_image_mime_types() - EXCLUDED_IMAGE_TYPES
